@@ -25,7 +25,6 @@
 #include <QImage>
 #include <QScopedPointer>
 
-#include "kis_shared.h"
 #include "kis_properties_configuration.h"
 #include <brushengine/kis_paint_information.h>
 #include <brushengine/kis_uniform_paintop_property.h>
@@ -48,14 +47,14 @@ class KisPaintopSettingsUpdateProxy;
  * property of KisPaintOpPreset. Whenever the settings are changed/modified from the original -- the preset is
  * set to dirty.
  */
-class KRITAIMAGE_EXPORT KisPaintOpSettings : public KisPropertiesConfiguration, public KisShared
+class KRITAIMAGE_EXPORT KisPaintOpSettings : public KisPropertiesConfiguration
 {
 
 public:
 
     KisPaintOpSettings();
-
     virtual ~KisPaintOpSettings();
+    KisPaintOpSettings(const KisPaintOpSettings &rhs);
 
     /**
      *
@@ -142,7 +141,7 @@ public:
      * Outline mode has to be passed to the paintop which builds the outline as some paintops have to paint outline
      * always like clone paintop indicating the duplicate position
      */
-    virtual QPainterPath brushOutline(const KisPaintInformation &info, OutlineMode mode) const;
+    virtual QPainterPath brushOutline(const KisPaintInformation &info, OutlineMode mode);
 
     /**
     * Helpers for drawing the brush outline
@@ -178,17 +177,17 @@ public:
     /**
      * @return opacity saved in the properties
      */
-    qreal paintOpOpacity() const;
+    qreal paintOpOpacity();
 
     /**
      * @return flow saved in the properties
      */
-    qreal paintOpFlow() const;
+    qreal paintOpFlow();
 
     /**
      * @return composite mode saved in the properties
      */
-    QString paintOpCompositeOp() const;
+    QString paintOpCompositeOp();
 
     /**
      * Set paintop size directly in the properties
@@ -201,14 +200,19 @@ public:
     virtual qreal paintOpSize() const = 0;
 
     void setEraserMode(bool value);
-    bool eraserMode() const;
+    bool eraserMode();
 
     qreal savedEraserSize() const;
     void setSavedEraserSize(qreal value);
     qreal savedBrushSize() const;
     void setSavedBrushSize(qreal value);
 
-    QString effectivePaintOpCompositeOp() const;
+    qreal savedEraserOpacity() const;
+    void setSavedEraserOpacity(qreal value);
+    qreal savedBrushOpacity() const;
+    void setSavedBrushOpacity(qreal value);
+
+    QString effectivePaintOpCompositeOp();
 
     void setPreset(KisPaintOpPresetWSP preset);
 
@@ -249,10 +253,10 @@ public:
      */
     void setProperty(const QString & name, const QVariant & value);
 
-    virtual QList<KisUniformPaintOpPropertySP> uniformProperties();
+    virtual QList<KisUniformPaintOpPropertySP> uniformProperties(KisPaintOpSettingsSP settings);
 
-    static bool isLodUserAllowed(const KisPropertiesConfiguration *config);
-    static void setLodUserAllowed(KisPropertiesConfiguration *config, bool value);
+    static bool isLodUserAllowed(const KisPropertiesConfigurationSP config);
+    static void setLodUserAllowed(KisPropertiesConfigurationSP config, bool value);
 
     /**
     * @return the option widget of the paintop (can be 0 is no option widgets is set)
