@@ -45,6 +45,9 @@
 #include "kis_command_utils.h"
 #include "kis_processing_applicator.h"
 #include "commands_new/kis_change_projection_color_command.h"
+#include "kis_layer_properties_icons.h"
+#include "lazybrush/kis_colorize_mask.h"
+#include "commands/kis_node_property_list_command.h"
 
 
 namespace KisLayerUtils {
@@ -144,6 +147,31 @@ namespace KisLayerUtils {
         MergeDownInfoBaseSP m_info;
     };
 
+    struct DisableColorizeKeyStrokes : public KisCommandUtils::AggregateCommand {
+        DisableColorizeKeyStrokes(MergeDownInfoBaseSP info) : m_info(info) {}
+
+        void populateChildCommands() {
+            Q_FOREACH (KisNodeSP node, m_info->allSrcNodes()) {
+                recursiveApplyNodes(node,
+                                    [this] (KisNodeSP node) {
+                                        if (dynamic_cast<KisColorizeMask*>(node.data()) &&
+                                            KisLayerPropertiesIcons::nodeProperty(node, KisLayerPropertiesIcons::colorizeEditKeyStrokes, true).toBool()) {
+
+                                            KisBaseNode::PropertyList props = node->sectionModelProperties();
+                                            KisLayerPropertiesIcons::setNodeProperty(&props,
+                                                                                     KisLayerPropertiesIcons::colorizeEditKeyStrokes,
+                                                                                     false);
+
+                                            addCommand(new KisNodePropertyListCommand(node, props));
+                                        }
+                                    });
+            }
+        }
+
+    private:
+        MergeDownInfoBaseSP m_info;
+    };
+
     struct RefreshHiddenAreas : public KUndo2Command {
         RefreshHiddenAreas(MergeDownInfoBaseSP info) : m_info(info) {}
 
@@ -244,6 +272,7 @@ namespace KisLayerUtils {
 
             if (m_info->frames.size() > 0) {
                 m_info->dstNode->enableAnimation();
+                m_info->dstNode->getKeyframeChannel(KisKeyframeChannel::Content.id(), true);
             }
         }
 
@@ -252,21 +281,30 @@ namespace KisLayerUtils {
     };
 
     struct CreateMergedLayerMultiple : public KisCommandUtils::AggregateCommand {
-        CreateMergedLayerMultiple(MergeMultipleInfoSP info) : m_info(info) {}
+        CreateMergedLayerMultiple(MergeMultipleInfoSP info, const QString name = QString() ) 
+            : m_info(info),
+              m_name(name) {}
 
         void populateChildCommands() {
-            const QString mergedLayerSuffix = i18n("Merged");
-            QString mergedLayerName = m_info->mergedNodes.first()->name();
+            QString mergedLayerName;
+            
+            if (m_name.isEmpty()){
+                const QString mergedLayerSuffix = i18n("Merged");
+                mergedLayerName = m_info->mergedNodes.first()->name();
 
-            if (!mergedLayerName.endsWith(mergedLayerSuffix)) {
-                mergedLayerName = QString("%1 %2")
-                    .arg(mergedLayerName).arg(mergedLayerSuffix);
+                if (!mergedLayerName.endsWith(mergedLayerSuffix)) {
+                    mergedLayerName = QString("%1 %2")
+                        .arg(mergedLayerName).arg(mergedLayerSuffix);
+                }
+            } else {
+                mergedLayerName = m_name;
             }
-
+                
             m_info->dstNode = new KisPaintLayer(m_info->image, mergedLayerName, OPACITY_OPAQUE_U8);
 
             if (m_info->frames.size() > 0) {
                 m_info->dstNode->enableAnimation();
+                m_info->dstNode->getKeyframeChannel(KisKeyframeChannel::Content.id(), true);
             }
 
             QString compositeOpId;
@@ -300,6 +338,7 @@ namespace KisLayerUtils {
 
     private:
         MergeMultipleInfoSP m_info;
+        QString m_name;
     };
 
     struct MergeLayers : public KisCommandUtils::AggregateCommand {
@@ -480,6 +519,30 @@ namespace KisLayerUtils {
         addCommand(cmd);
     }
 
+    struct InsertNode : public KisCommandUtils::AggregateCommand {
+        InsertNode(MergeDownInfoBaseSP info, KisNodeSP putAfter)
+            : m_info(info), m_putAfter(putAfter) {}
+        
+        void populateChildCommands() {
+            addCommand(new KisImageLayerAddCommand(m_info->image,
+                                                           m_info->dstNode,
+                                                           m_putAfter->parent(),
+                                                           m_putAfter,
+                                                           true, false));
+        
+        }
+
+    private:
+        virtual void addCommandImpl(KUndo2Command *cmd) {
+            addCommand(cmd);
+        }
+
+    private:
+        MergeDownInfoBaseSP m_info;
+        KisNodeSP m_putAfter;
+    };
+
+
     struct CleanUpNodes : private RemoveNodeHelper, public KisCommandUtils::AggregateCommand {
         CleanUpNodes(MergeDownInfoBaseSP info, KisNodeSP putAfter)
             : m_info(info), m_putAfter(putAfter) {}
@@ -675,6 +738,11 @@ namespace KisLayerUtils {
         if (layer->visible() && prevLayer->visible()) {
             MergeDownInfoSP info(new MergeDownInfo(image, prevLayer, layer));
 
+            // disable key strokes on all colorize masks and wait until
+            // update is finished with a barrier
+            applicator.applyCommand(new DisableColorizeKeyStrokes(info));
+            applicator.applyCommand(new KUndo2Command(), KisStrokeJobData::BARRIER);
+
             applicator.applyCommand(new KeepMergedNodesSelected(info, false));
             applicator.applyCommand(new FillSelectionMasks(info));
             applicator.applyCommand(new CreateMergedLayer(info), KisStrokeJobData::BARRIER);
@@ -922,7 +990,9 @@ namespace KisLayerUtils {
         applicator.end();
     }
 
-    void mergeMultipleLayersImpl(KisImageSP image, KisNodeList mergedNodes, KisNodeSP putAfter, bool flattenSingleLayer, const KUndo2MagicString &actionName)
+    void mergeMultipleLayersImpl(KisImageSP image, KisNodeList mergedNodes, KisNodeSP putAfter, 
+                                           bool flattenSingleLayer, const KUndo2MagicString &actionName, 
+                                           bool cleanupNodes = true, const QString layerName = QString())
     {
         filterMergableNodes(mergedNodes);
         {
@@ -959,9 +1029,14 @@ namespace KisLayerUtils {
         if (mergedNodes.size() > 1 || invisibleNodes.isEmpty()) {
             MergeMultipleInfoSP info(new MergeMultipleInfo(image, mergedNodes));
 
+            // disable key strokes on all colorize masks and wait until
+            // update is finished with a barrier
+            applicator.applyCommand(new DisableColorizeKeyStrokes(info));
+            applicator.applyCommand(new KUndo2Command(), KisStrokeJobData::BARRIER);
+
             applicator.applyCommand(new KeepMergedNodesSelected(info, putAfter, false));
             applicator.applyCommand(new FillSelectionMasks(info));
-            applicator.applyCommand(new CreateMergedLayerMultiple(info), KisStrokeJobData::BARRIER);
+            applicator.applyCommand(new CreateMergedLayerMultiple(info, layerName), KisStrokeJobData::BARRIER);
 
             if (info->frames.size() > 0) {
                 foreach (int frame, info->frames) {
@@ -979,9 +1054,15 @@ namespace KisLayerUtils {
             }
 
             //applicator.applyCommand(new MergeMetaData(info, strategy), KisStrokeJobData::BARRIER);
-            applicator.applyCommand(new CleanUpNodes(info, putAfter),
-                                    KisStrokeJobData::SEQUENTIAL,
-                                    KisStrokeJobData::EXCLUSIVE);
+            if (cleanupNodes){
+                applicator.applyCommand(new CleanUpNodes(info, putAfter),
+                                            KisStrokeJobData::SEQUENTIAL,
+                                        KisStrokeJobData::EXCLUSIVE);
+            } else {
+                applicator.applyCommand(new InsertNode(info, putAfter),
+                                            KisStrokeJobData::SEQUENTIAL,
+                                        KisStrokeJobData::EXCLUSIVE);
+            }
             applicator.applyCommand(new KeepMergedNodesSelected(info, putAfter, true));
         }
 
@@ -991,7 +1072,15 @@ namespace KisLayerUtils {
 
     void mergeMultipleLayers(KisImageSP image, KisNodeList mergedNodes, KisNodeSP putAfter)
     {
-        mergeMultipleLayersImpl(image, mergedNodes, putAfter, false, kundo2_i18n("Merge Selected Nodes"));
+        mergeMultipleLayersImpl(image, mergedNodes, putAfter, false, kundo2_i18n("Merge Selected Nodes"), false);
+    }
+    
+    void newLayerFromVisible(KisImageSP image, KisNodeSP putAfter)
+    {
+        KisNodeList mergedNodes;
+        mergedNodes << image->root();
+
+        mergeMultipleLayersImpl(image, mergedNodes, putAfter, true, kundo2_i18n("New From Visible"), false, i18nc("New layer created from all the visible layers", "Visible"));
     }
 
     struct MergeSelectionMasks : public KisCommandUtils::AggregateCommand {
