@@ -18,6 +18,7 @@
 
 #include "kis_colorize_mask.h"
 
+#include <mutex>
 #include <QCoreApplication>
 
 #include <KoColorSpaceRegistry.h>
@@ -57,7 +58,7 @@ struct KisColorizeMask::Private
           showColoring(true),
           needsUpdate(true),
           originalSequenceNumber(-1),
-          updateCompressor(1000, KisSignalCompressor::POSTPONE)
+          updateCompressor(1, KisSignalCompressor::POSTPONE)
     {
     }
 
@@ -488,6 +489,11 @@ QRect KisColorizeMask::exactBounds() const
 
 }
 
+QRect KisColorizeMask::nonDependentExtent() const
+{
+    return extent();
+}
+
 KisImageSP KisColorizeMask::fetchImage() const
 {
     KisLayerSP parentLayer = dynamic_cast<KisLayer*>(parent().data());
@@ -515,7 +521,8 @@ void KisColorizeMask::setCurrentColor(const KoColor &_color)
     KoColor color = _color;
     color.convertTo(colorSpace());
 
-    lockTemporaryTargetForWrite();
+    WriteLockableWrapper lock(this);
+    std::lock_guard<WriteLockableWrapper> guard(lock);
 
     setNeedsUpdate(true);
 
@@ -541,8 +548,6 @@ void KisColorizeMask::setCurrentColor(const KoColor &_color)
     m_d->currentColor = color;
     m_d->currentKeyStrokeDevice = activeDevice;
     m_d->needAddCurrentKeyStroke = newKeyStroke;
-
-    unlockTemporaryTarget();
 }
 
 
@@ -575,6 +580,9 @@ void KisColorizeMask::mergeToLayer(KisNodeSP layer, KisPostExecutionUndoAdapter
 {
     Q_UNUSED(layer);
 
+    WriteLockableWrapper lock(this);
+    std::lock_guard<WriteLockableWrapper> guard(lock);
+
     KisPaintDeviceSP temporaryTarget = this->temporaryTarget();
     const bool isTemporaryTargetErasing = temporaryCompositeOp() == COMPOSITE_ERASE;
     const QRect temporaryExtent = temporaryTarget ? temporaryTarget->extent() : QRect();
@@ -620,8 +628,6 @@ void KisColorizeMask::mergeToLayer(KisNodeSP layer, KisPostExecutionUndoAdapter
      * Try removing the key strokes that has been completely erased
      */
     if (isTemporaryTargetErasing) {
-        lockTemporaryTargetForWrite();
-
         for (int index = 0; index < m_d->keyStrokes.size(); /*noop*/) {
             const KeyStroke &stroke = m_d->keyStrokes[index];
 
@@ -637,8 +643,6 @@ void KisColorizeMask::mergeToLayer(KisNodeSP layer, KisPostExecutionUndoAdapter
                 index++;
             }
         }
-
-        unlockTemporaryTarget();
     }
 
     undoAdapter->addMacro(macro);
