@@ -287,12 +287,12 @@ protected:
     KisImageWSP currentImage();
     KoPattern* currentPattern();
     KoAbstractGradient *currentGradient();
-    KisNodeSP currentNode();
+    KisNodeSP currentNode() const;
     KisNodeList selectedNodes() const;
     KoColor currentFgColor();
     KoColor currentBgColor();
     KisPaintOpPresetSP currentPaintOpPreset();
-    KisFilterConfiguration *currentGenerator();
+    KisFilterConfigurationSP currentGenerator();
 
     virtual void setupPaintAction(KisRecordedPaintAction* action);
 
