@@ -22,7 +22,9 @@
 
 #include "kis_canvas2.h"
 #include "kis_image.h"
+#include <QMenu>
 #include <kis_icon.h>
+#include <kis_transform_mask.h>
 #include "KisViewManager.h"
 #include "kis_action_manager.h"
 #include "kis_image_animation_interface.h"
@@ -36,6 +38,8 @@
 #include "kis_image_config.h"
 #include "kis_config.h"
 #include "kis_signals_blocker.h"
+#include "kis_node_manager.h"
+#include "kis_transform_mask_params_factory_registry.h"
 
 
 #include "ui_wdg_animation.h"
@@ -64,7 +68,6 @@ AnimationDocker::AnimationDocker()
 
     m_animationWidget->setupUi(mainWidget);
 
-
     m_previousFrameAction = new KisAction(i18n("Previous Frame"), m_animationWidget->btnPreviousFrame);
     m_previousFrameAction->setActivationFlags(KisAction::ACTIVE_IMAGE);
     m_animationWidget->btnPreviousFrame->setDefaultAction(m_previousFrameAction);
@@ -94,17 +97,31 @@ AnimationDocker::AnimationDocker()
     m_animationWidget->btnPlay->setDefaultAction(m_playPauseAction);
 
     m_addBlankFrameAction = new KisAction(KisAnimationUtils::addFrameActionName, m_animationWidget->btnAddKeyframe);
-    m_addBlankFrameAction->setActivationFlags(KisAction::ACTIVE_LAYER);
+    m_addBlankFrameAction->setActivationFlags(KisAction::ACTIVE_NODE);
     m_animationWidget->btnAddKeyframe->setDefaultAction(m_addBlankFrameAction);
 
     m_addDuplicateFrameAction = new KisAction(KisAnimationUtils::duplicateFrameActionName, m_animationWidget->btnAddDuplicateFrame);
-    m_addDuplicateFrameAction->setActivationFlags(KisAction::ACTIVE_LAYER);
+    m_addDuplicateFrameAction->setActivationFlags(KisAction::ACTIVE_DEVICE);
     m_animationWidget->btnAddDuplicateFrame->setDefaultAction(m_addDuplicateFrameAction);
 
     m_deleteKeyframeAction = new KisAction(KisAnimationUtils::removeFrameActionName, m_animationWidget->btnDeleteKeyframe);
-    m_deleteKeyframeAction->setActivationFlags(KisAction::ACTIVE_LAYER);
+    m_deleteKeyframeAction->setActivationFlags(KisAction::ACTIVE_NODE);
     m_animationWidget->btnDeleteKeyframe->setDefaultAction(m_deleteKeyframeAction);
 
+    m_newKeyframeMenu = new QMenu(this);
+    m_animationWidget->btnAddKeyframe->setMenu(m_newKeyframeMenu);
+    m_animationWidget->btnAddKeyframe->setPopupMode(QToolButton::MenuButtonPopup);
+
+    m_deleteKeyframeMenu = new QMenu(this);
+    m_animationWidget->btnDeleteKeyframe->setMenu(m_deleteKeyframeMenu);
+    m_animationWidget->btnDeleteKeyframe->setPopupMode(QToolButton::MenuButtonPopup);
+
+    m_addOpacityKeyframeAction = new KisAction(KisAnimationUtils::addOpacityKeyframeActionName);
+    m_deleteOpacityKeyframeAction = new KisAction(KisAnimationUtils::removeOpacityKeyframeActionName);
+
+    m_addTransformKeyframeAction = new KisAction(KisAnimationUtils::addTransformKeyframeActionName);
+    m_deleteTransformKeyframeAction = new KisAction(KisAnimationUtils::removeTransformKeyframeActionName);
+
     {
         KisImageConfig cfg;
         setupActionButton(KisAnimationUtils::lazyFrameCreationActionName,
@@ -145,6 +162,12 @@ AnimationDocker::AnimationDocker()
     connect(m_lazyFrameAction, SIGNAL(toggled(bool)), this, SLOT(slotLazyFrameChanged(bool)));
     connect(m_dropFramesAction, SIGNAL(toggled(bool)), this, SLOT(slotDropFramesChanged(bool)));
 
+    connect(m_addOpacityKeyframeAction, SIGNAL(triggered(bool)), this, SLOT(slotAddOpacityKeyframe()));
+    connect(m_deleteOpacityKeyframeAction, SIGNAL(triggered(bool)), this, SLOT(slotDeleteOpacityKeyframe()));
+
+    connect(m_addTransformKeyframeAction, SIGNAL(triggered(bool)), this, SLOT(slotAddTransformKeyframe()));
+    connect(m_deleteTransformKeyframeAction, SIGNAL(triggered(bool)), this, SLOT(slotDeleteTransformKeyframe()));
+
     m_animationWidget->btnOnionSkinOptions->setToolTip(i18n("Onion Skins"));
     connect(m_animationWidget->btnOnionSkinOptions, SIGNAL(clicked()), this, SLOT(slotOnionSkinOptions()));
 
@@ -172,6 +195,7 @@ void AnimationDocker::setCanvas(KoCanvasBase * canvas)
         m_canvas->image()->disconnect(this);
         m_canvas->image()->animationInterface()->disconnect(this);
         m_canvas->animationPlayer()->disconnect(this);
+        m_canvas->viewManager()->nodeManager()->disconnect(this);
     }
 
     m_canvas = dynamic_cast<KisCanvas2*>(canvas);
@@ -198,7 +222,11 @@ void AnimationDocker::setCanvas(KoCanvasBase * canvas)
                 m_canvas->animationPlayer(),
                 SLOT(slotUpdatePlaybackSpeed(double)));
 
+        connect(m_canvas->viewManager()->nodeManager(), SIGNAL(sigNodeActivated(KisNodeSP)),
+                this, SLOT(slotCurrentNodeChanged(KisNodeSP)));
+
         slotGlobalTimeChanged();
+        slotCurrentNodeChanged(m_canvas->viewManager()->nodeManager()->activeNode());
     }
 }
 
@@ -237,35 +265,48 @@ void AnimationDocker::setMainWindow(KisViewManager *view)
 
 void AnimationDocker::slotAddBlankFrame()
 {
-    if (!m_canvas) return;
-
-    KisNodeSP node = m_canvas->viewManager()->activeNode();
-    if (!node) return;
-
-    const int time = m_canvas->image()->animationInterface()->currentTime();
-    KisAnimationUtils::createKeyframeLazy(m_canvas->image(), node, time, false);
+    addKeyframe(KisKeyframeChannel::Content.id(), false);
 }
 
 void AnimationDocker::slotAddDuplicateFrame()
 {
-    if (!m_canvas) return;
-
-    KisNodeSP node = m_canvas->viewManager()->activeNode();
-    if (!node) return;
-
-    const int time = m_canvas->image()->animationInterface()->currentTime();
-    KisAnimationUtils::createKeyframeLazy(m_canvas->image(), node, time, true);
+    addKeyframe(KisKeyframeChannel::Content.id(), true);
 }
 
 void AnimationDocker::slotDeleteKeyframe()
+{
+    deleteKeyframe(KisKeyframeChannel::Content.id());
+}
+
+void AnimationDocker::slotAddOpacityKeyframe()
+{
+    addKeyframe(KisKeyframeChannel::Opacity.id(), false);
+}
+
+void AnimationDocker::slotDeleteOpacityKeyframe()
+{
+    deleteKeyframe(KisKeyframeChannel::Opacity.id());
+}
+
+void AnimationDocker::slotAddTransformKeyframe()
 {
     if (!m_canvas) return;
 
-    KisNodeSP node = m_canvas->viewManager()->activeNode();
-    if (!node) return;
+    KisTransformMask *mask = dynamic_cast<KisTransformMask*>(m_canvas->viewManager()->activeNode().data());
+    if (!mask) return;
 
     const int time = m_canvas->image()->animationInterface()->currentTime();
-    KisAnimationUtils::removeKeyframe(m_canvas->image(), node, time);
+
+    KUndo2Command *command = new KUndo2Command(kundo2_i18n("Add transform keyframe"));
+    KisTransformMaskParamsFactoryRegistry::instance()->autoAddKeyframe(mask, time, KisTransformMaskParamsInterfaceSP(), command);
+
+    command->redo();
+    m_canvas->currentImage()->postExecutionUndoAdapter()->addCommand(toQShared(command));
+}
+
+void AnimationDocker::slotDeleteTransformKeyframe()
+{
+    deleteKeyframe(KisKeyframeChannel::TransformArguments.id());
 }
 
 void AnimationDocker::slotUIRangeChanged()
@@ -564,3 +605,59 @@ void AnimationDocker::slotDropFramesChanged(bool value)
         updateDropFramesIcon();
     }
 }
+
+void AnimationDocker::slotCurrentNodeChanged(KisNodeSP node)
+{
+    bool isNodeAnimatable = false;
+
+    m_newKeyframeMenu->clear();
+    m_deleteKeyframeMenu->clear();
+
+    if (!node.isNull()) {
+        if (KisAnimationUtils::supportsContentFrames(node)) {
+            isNodeAnimatable = true;
+            m_newKeyframeMenu->addAction(m_addBlankFrameAction);
+            m_deleteKeyframeMenu->addAction(m_deleteKeyframeAction);
+        }
+
+        if (node->inherits("KisLayer")) {
+            isNodeAnimatable = true;
+            m_newKeyframeMenu->addAction(m_addOpacityKeyframeAction);
+            m_deleteKeyframeMenu->addAction(m_deleteOpacityKeyframeAction);
+        }
+
+        /*
+        if (node->inherits("KisTransformMask")) {
+            isNodeAnimatable = true;
+            m_newKeyframeMenu->addAction(m_addTransformKeyframeAction);
+            m_deleteKeyframeMenu->addAction(m_deleteTransformKeyframeAction);
+        }
+        */
+    }
+
+    m_animationWidget->btnAddKeyframe->setEnabled(isNodeAnimatable);
+    m_animationWidget->btnAddDuplicateFrame->setEnabled(isNodeAnimatable);
+    m_animationWidget->btnDeleteKeyframe->setEnabled(isNodeAnimatable);
+}
+
+void AnimationDocker::addKeyframe(const QString &channel, bool copy)
+{
+    if (!m_canvas) return;
+
+    KisNodeSP node = m_canvas->viewManager()->activeNode();
+    if (!node) return;
+
+    const int time = m_canvas->image()->animationInterface()->currentTime();
+    KisAnimationUtils::createKeyframeLazy(m_canvas->image(), node, channel, time, copy);
+}
+
+void AnimationDocker::deleteKeyframe(const QString &channel)
+{
+    if (!m_canvas) return;
+
+    KisNodeSP node = m_canvas->viewManager()->activeNode();
+    if (!node) return;
+
+    const int time = m_canvas->image()->animationInterface()->currentTime();
+    KisAnimationUtils::removeKeyframe(m_canvas->image(), node, channel, time);
+}
